require 'anndb_api/util'

module AnndbApi
    class DatasetOpError < RuntimeError
    end

    class Dataset

        def delete(id)
            format_dataset_op_result_single @stub.delete(AnndbApiPb::DeleteRequest.new(ids: [
                AnndbApiPb::UUID.new(data: AnndbApi::Util.uuid_string_to_bytes(id))
                ]), { metadata: @rpc_metadata })
        end

        def delete_batch(ids)
            format_dataset_op_result @stub.delete(AnndbApiPb::DeleteRequest.new(ids: ids.map { |id|
                AnndbApiPb::UUID.new(data: AnndbApi::Util.uuid_string_to_bytes(id))
                }), { metadata: @rpc_metadata })
        end

        protected

        def format_search_result(result)
            result.items.map { |item|
                {
                    id: AnndbApi::Util.ascii8bit_to_uuid(item.id),
                    metadata: item.metadata
                }
            }
        end

        def format_dataset_op_result(result)
            (0..result.ids.length-1).map { |i|
                {
                    id: AnndbApi::Util.ascii8bit_to_uuid(result.ids[i].data),
                    error: result.errors[i].message
                }
            }
        end

        def format_dataset_op_result_single(result)
            error = result.errors.first.message
            if error.length > 0
                raise AnndbApi::DatasetOpError.new(error)
            end

            AnndbApi::Util.ascii8bit_to_uuid(result.ids.first.data)
        end

        def format_req_metadata(metadata)
            metadata.collect{ |k, v| [k.to_s, v] }.to_h
        end
    end
end