require 'grpc'

require 'anndb_api/dataset'
require 'core_pb'
require 'image_dataset_pb'
require 'image_dataset_services_pb'

module AnndbApi
    class ImageDataset < AnndbApi::Dataset
        def initialize(server_addr, rpc_metadata)
            @rpc_metadata = rpc_metadata
            @stub = AnndbApiPb::ImageDataset::Stub.new(server_addr, GRPC::Core::ChannelCredentials.new())
        end

        def insert(image, metadata = {})
            format_dataset_op_result_single @stub.insert(AnndbApiPb::ImageDatasetRequest.new(items: [
                AnndbApiPb::ImageItem.new(
                    image: image_proto(image),
                    metadata: format_req_metadata(metadata)
                    )
                ]), { metadata: @rpc_metadata })
        end

        def insert_batch(items)
            format_dataset_op_result @stub.insert(AnndbApiPb::ImageDatasetRequest.new(items: items.map { |item|
                AnndbApiPb::ImageItem.new(
                    image: image_proto(item[:image]),
                    metadata: format_req_metadata(item.fetch(:metadata, {}))
                    )
                }), { metadata: @rpc_metadata })
        end

        def update(id, image, metadata = {})
            format_dataset_op_result_single @stub.update(AnndbApiPb::ImageDatasetRequest.new(items: [
                AnndbApiPb::ImageItem.new(
                    id: AnndbApiPb::UUID.new(data: AnndbApi::Util.uuid_string_to_bytes(id)),
                    image: image_proto(image),
                    metadata: format_req_metadata(metadata)
                    )
                ]), { metadata: @rpc_metadata })
        end

        def update_batch(items)
            format_dataset_op_result @stub.update(AnndbApiPb::ImageDatasetRequest.new(items: items.map { |item|
                AnndbApiPb::ImageItem.new(
                    id: AnndbApiPb::UUID.new(data: AnndbApi::Util.uuid_string_to_bytes(item[:id])),
                    image: image_proto(item[:image]),
                    metadata: format_req_metadata(item.fetch(:metadata, {}))
                    )
                }), { metadata: @rpc_metadata })
        end

        def search_image(query, n)
            format_search_result @stub.search_image(AnndbApiPb::ImageSearchImageRequest.new(
                query: image_proto(query),
                n: n.to_i
                ), { metadata: @rpc_metadata })
        end

        def search_text(query, n)
            format_search_result @stub.search_text(AnndbApiPb::ImageSearchTextRequest.new(
                query: query.to_s,
                n: n.to_i
                ), { metadata: @rpc_metadata })
        end

        private

        def image_proto(img)
            if !img.match(/^http/).nil?
                return AnndbApiPb::Image.new(url: img)
            else
                return AnndbApiPb::Image.new(data: img)
            end
            
            raise TypeError.new('Image has to be either an array or a string')
        end
    end
end