require 'grpc'

require 'anndb_api/dataset'
require 'text_dataset_pb'
require 'text_dataset_services_pb'

module AnndbApi
    class TextDataset < AnndbApi::Dataset
        def initialize(server_addr, rpc_metadata)
            @rpc_metadata = rpc_metadata
            @stub = AnndbApiPb::TextDataset::Stub.new(server_addr, GRPC::Core::ChannelCredentials.new())
        end

        def insert(text, metadata = {})
            format_dataset_op_result_single @stub.insert(AnndbApiPb::TextDatasetRequest.new(items: [
                AnndbApiPb::TextItem.new(
                    text: text,
                    metadata: format_req_metadata(metadata)
                    )
                ]), { metadata: @rpc_metadata })
        end

        def insert_batch(items)
            format_dataset_op_result @stub.insert(AnndbApiPb::TextDatasetRequest.new(items: items.map { |item|
                AnndbApiPb::TextItem.new(
                    text: item[:text],
                    metadata: format_req_metadata(item.fetch(:metadata, {}))
                    )

                }), { metadata: @rpc_metadata })
        end

        def update(id, text, metadata = {})
            format_dataset_op_result_single @stub.update(AnndbApiPb::TextDatasetRequest.new(items: [
                AnndbApiPb::TextItem.new(
                    id: AnndbApiPb::UUID.new(data: AnndbApi::Util.uuid_string_to_bytes(id)),
                    text: text,
                    metadata: format_req_metadata(metadata)
                    )
                ]), { metadata: @rpc_metadata })
        end

        def update_batch(items)
            format_dataset_op_result @stub.update(AnndbApiPb::TextDatasetRequest.new(items: items.map { |item|
                AnndbApiPb::TextItem.new(
                    id: AnndbApiPb::UUID.new(data: AnndbApi::Util.uuid_string_to_bytes(item[:id])),
                    text: item[:text],
                    metadata: format_req_metadata(item.fetch(:metadata, {}))
                    )
                }), { metadata: @rpc_metadata })
        end

        def search(query, n)
            format_search_result @stub.search(AnndbApiPb::TextSearchRequest.new(query: query, n: n), { metadata: @rpc_metadata })
        end
    end
end