require 'grpc'

require 'anndb_api/dataset'
require 'core_pb'
require 'vector_dataset_pb'
require 'vector_dataset_services_pb'

module AnndbApi
    class VectorDataset < AnndbApi::Dataset
        def initialize(server_addr, rpc_metadata)
            @rpc_metadata = rpc_metadata
            @stub = AnndbApiPb::VectorDataset::Stub.new(server_addr, GRPC::Core::ChannelCredentials.new())
        end

        def insert(vector, metadata = {})
            format_dataset_op_result_single @stub.insert(AnndbApiPb::VectorDatasetRequest.new(items: [
                AnndbApiPb::VectorItem.new(
                    vector: AnndbApiPb::Vector.new(values: vector),
                    metadata: format_req_metadata(metadata)
                    )
                ]), { metadata: @rpc_metadata })
        end

        def insert_batch(items)
            format_dataset_op_result @stub.insert(AnndbApiPb::VectorDatasetRequest.new(items: items.map { |item|
                AnndbApiPb::VectorItem.new(
                    vector: AnndbApiPb::Vector.new(values: item[:vector]),
                    metadata: format_req_metadata(item.fetch(:metadata, {}))
                    )

                }), { metadata: @rpc_metadata })
        end

        def update(id, vector, metadata = {})
            format_dataset_op_result_single @stub.update(AnndbApiPb::VectorDatasetRequest.new(items: [
                AnndbApiPb::VectorItem.new(
                    id: AnndbApiPb::UUID.new(data: AnndbApi::Util.uuid_string_to_bytes(id)),
                    vector: AnndbApiPb::Vector.new(values: vector),
                    metadata: format_req_metadata(metadata)
                    )
                ]), { metadata: @rpc_metadata })
        end

        def update_batch(items)
            format_dataset_op_result @stub.update(AnndbApiPb::VectorDatasetRequest.new(items: items.map { |item|
                AnndbApiPb::VectorItem.new(
                    id: AnndbApiPb::UUID.new(data: AnndbApi::Util.uuid_string_to_bytes(item[:id])),
                    vector: AnndbApiPb::Vector.new(values: item[:vector]),
                    metadata: format_req_metadata(item.fetch(:metadata, {}))
                    )
                }), { metadata: @rpc_metadata })
        end

        def search(query, n)
            format_search_result @stub.search(AnndbApiPb::VectorSearchRequest.new(
                query: AnndbApiPb::Vector.new(values: query),
                n: n
                ), { metadata: @rpc_metadata })
        end
    end
end